require 'ostruct'

SLVector3 = Struct.new(:x, :y, :z)
SLQuaternion = Struct.new(:x, :y, :z, :s)
SLRegion = Struct.new(:name, :x, :y)
SLOwner = Struct.new(:name, :key)

module Rack
  class SL

    def initialize(app)
      @app = app
    end

    def call(env)
      sl = env['HTTP_X_SECONDLIFE_REGION'] ? parse_headers(env) : {from_sl: false}
      env['sl'] = OpenStruct.new(sl)
      @app.call(env)
    end

    private

    def parse_headers(env)
      begin
        sl              = {from_sl: true}
        sl[:shard]      = env['HTTP_X_SECONDLIFE_SHARD']
        sl[:name]       = env['HTTP_X_SECONDLIFE_OBJECT_NAME']
        sl[:key]        = env['HTTP_X_SECONDLIFE_OBJECT_KEY']
        sl[:region]     = parse_region(env['HTTP_X_SECONDLIFE_REGION'])
        sl[:position]   = parse_vector3(env['HTTP_X_SECONDLIFE_LOCAL_POSITION'])
        sl[:rotation]   = parse_rotation(env['HTTP_X_SECONDLIFE_LOCAL_ROTATION'])
        sl[:velocity]   = parse_vector3(env['HTTP_X_SECONDLIFE_LOCAL_VELOCITY'])
        sl[:owner]      = SLOwner.new(env['HTTP_X_SECONDLIFE_OWNER_NAME'], 
                                      env['HTTP_X_SECONDLIFE_OWNER_KEY'])
        sl
      rescue => e
        {from_sl: false, error: e}
      end
    end

    def parse_region(value)
      name = /^.+(?= \()/.match(value).to_s
      x, y = value.match(/\((\-?[0-9]+), (\-?[0-9]+)\)/).captures
      SLRegion.new(name, x.to_i, y.to_i)
    end

    def parse_vector3(value)
      x, y, z = value.scan(/\-?\d+.\d+/)
      SLVector3.new(x.to_f, y.to_f, z.to_f)
    end

    def parse_rotation(value)
      x, y, z, s = value.scan(/\-?\d+.\d+/)
      SLQuaternion.new(x.to_f, y.to_f, z.to_f, s.to_f)
    end
  end
end
